; ================================================================================================
; SW-PWM-Routine fr 24 Kanle an 3 Ports
; holt Bytes aus dem RAM und gibt SW-PWM an den eingestellten Ports aus
; ------------------------------------------------------------------------------------------------
;
; benutzte Register      : temp0, ZL, ZH, werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : pwm_count
;
; Konstanten (zum Kopieren):
;
;.def pwm_count 			= R23				; (>15) Zhler fr PWM
;
;.equ PWM_BUFFER		= 0x0060			; RAM-Puffer fr PWM-Ausgabedaten
;
; ================================================================================================

_sw_pwm:

		push	temp0						; Register sichern
		push	temp1
		push	temp2
		push	temp3
		push	ZL
		push	ZH

		inc		pwm_count					; PWM-Zhler weiter
		cpi		pwm_count, 255				; komplett durch?
		brlo	_sw_pwm_weiter				; nein, weiter
		clr		pwm_count					; ja, zurck setzen

_sw_pwm_weiter:

		ldi		ZL, low(PWM_BUFFER)			; Pointer auf RAM-Adresse Puffer setzen
		ldi		ZH, high(PWM_BUFFER)

		ser		temp1						; Ausgaberegister auf Grundzustand "LED an"
		ser		temp2
		ser		temp3

; ------------------------------------------------------------------------------------------------

_sw_pwm_1:

        ld      temp0, Z+               	; Wert fr Kanal 1 aus dem RAM holen
        cp      pwm_count, temp0        	; Ist der Grenzwert fr Kanal 1 erreicht?
        brlo    _sw_pwm_2                   ; nein, weiter zu Kanal 2
        cbr	    temp1, (1<<0)       		; ja, LED Kanal 1 aus (hier: PortA0)

_sw_pwm_2:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_3
        cbr	    temp1, (1<<1)

_sw_pwm_3:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_4
        cbr	    temp1, (1<<2)

_sw_pwm_4:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_5
        cbr	    temp1, (1<<3)

_sw_pwm_5:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_6
        cbr	    temp1, (1<<4)

_sw_pwm_6:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_7
        cbr	    temp1, (1<<5)

_sw_pwm_7:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_8
        cbr	    temp1, (1<<6)

_sw_pwm_8:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_9
        cbr	    temp1, (1<<7)

_sw_pwm_9:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_10
        cbr	    temp2, (1<<0)

_sw_pwm_10:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_11
        cbr	    temp2, (1<<1)

_sw_pwm_11:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_12
        cbr	    temp2, (1<<2)

_sw_pwm_12:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_13
        cbr	    temp2, (1<<3)

_sw_pwm_13:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_14
        cbr	    temp2, (1<<4)

_sw_pwm_14:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_15
        cbr	    temp2, (1<<5)

_sw_pwm_15:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_16
        cbr	    temp2, (1<<6)

_sw_pwm_16:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_17
        cbr	    temp2, (1<<7)

_sw_pwm_17:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_18
        cbr	    temp3, (1<<0)

_sw_pwm_18:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_19
        cbr	    temp3, (1<<1)

_sw_pwm_19:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_20
        cbr	    temp3, (1<<2)

_sw_pwm_20:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_21
        cbr	    temp3, (1<<3)

_sw_pwm_21:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_22
        cbr	    temp3, (1<<4)

_sw_pwm_22:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_23
        cbr	    temp3, (1<<5)

_sw_pwm_23:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_24
        cbr	    temp3, (1<<6)

_sw_pwm_24:

        ld      temp0, Z+               	; usw.
        cp      pwm_count, temp0
        brlo    _sw_pwm_out
        cbr	    temp3, (1<<7)

; ------------------------------------------------------------------------------------------------

_sw_pwm_out:

		out		PortA, temp1				; Bitmuster ausgeben
		out		PortB, temp2
		out		PortC, temp3

		pop		ZH							; Register wiederherstellen
		pop		ZL
		pop		temp3
		pop		temp2
		pop		temp1
		pop		temp0

		ret									; und raus

; ================================================================================================
