; ================================================================================================
; Mini-DMX-Empfangsroutine fr 96-512 Kanle (Festgelegt durch Header-Byte)
; Alle empfangenen Kanle werden gespeichert, Startadresse 1
; Besttigung wird in _ISR_Byte_Transmitted.inc im Interrupt versendet
;
; by Pesi - V 1.0 - 25.12.2010
; ================================================================================================
; benutzte Register      : temp0, temp1, temp2, ZL, ZH, werden gesichert
; gem. benutzte Register : tx_State
; reservierte Register   : siehe Definitionen
;
; Definitionen (zum Kopieren):
;
;.def mDMX_State			= R20				; (>15) Status mDMX-Empfangsroutine
;.def MDMX_Framecount_L	= R21				; (>15) Zhler fr empfangene Bytes L
;.def MDMX_Framecount_H	= R22				; (>15) Zhler fr empfangene Bytes H
;.def MDMX_Buffer_L		= R2				; Zeiger auf Empfangspuffer L
;.def MDMX_Buffer_H		= R3				; Zeiger auf Empfangspuffer H
;.def Blink 				= R23				; (>15) Zhler fr Blinktakt Status-LED
; 
;.equ MDMX_Buffer	= 0x0100				; RAM-Bereich fr empfangene Daten
; ------------------------------------------------------------------------------------------------

_USB_Received:

		push	temp1							; Register sichern
		in		temp1, SREG
		push	temp1
		push	temp0
		push	ZL
		push	ZH

		lds		temp1,UDR0						; empfangenes Byte in temp1

.include "Check_Bootstring.inc"					; auf Bootloader-Startcode prfen

		cpi		MDMX_State, 0					; 0 = warten auf Blockstart 0x5A
		breq	_MDMX_Blockstart

		cpi		MDMX_State, 101					; 101 = warten auf Framesize-Byte
		breq	_MDMX_Header

		cpi		MDMX_State, 102					; 102 = Daten speichern
		breq	_ISR_MDMX_Store_Byte					

		cpi		MDMX_State, 103					; 103 = warten auf Blockende 0xA5
		brne	_MDMX_State_Select_End
		rjmp	_MDMX_Blockende

_MDMX_State_Select_End:

		rjmp	_ISR_MDMX_Ende					; Zustand undefiniert oder Bootloader-Check, Ende

; ------------------------------------------------------------------------------------------------
; 0 = Warten auf Blockstart
; ------------------------------------------------------------------------------------------------

_MDMX_Blockstart:

		cpi		temp1, 0x5A						; Blockstart-Byte empfangen...?
		breq	_MDMX_Blockstart_OK				; ja, weiter
		rjmp	_ISR_MDMX_Clear					; nein, Receiver Reset

_MDMX_Blockstart_OK:

		ldi		MDMX_State, 101					; nchster Aufruf: auf Header warten

		ldi		temp1, (1<<TXEN1)			 	; Tx enable
		sts		UCSR1B, temp1

		rjmp	_ISR_MDMX_Ende					; und raus

; ------------------------------------------------------------------------------------------------
; 101 = Header auswerten - 0xA0 = 96 Kanle, 0xA1 = 256 Kan., 0xA2 = 512 Kan.
; ------------------------------------------------------------------------------------------------

_MDMX_Header:

		cpi		temp1, 0xA0						; mDMX-Header-Byte 96 Kanle empfangen...?
		brne	_MDMX_Header_256				; nein, auf 256 Kanle-Modus prfen
		
		ldi		MDMX_Framecount_L, low(96)		; ja, Framecounter auf 96 setzen
		ldi		MDMX_Framecount_H, high(96)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_256:

		cpi		temp1, 0xA1						; mDMX-Header-Byte 256 Kanle empfangen...?
		brne	_MDMX_Header_512				; nein, auf 512 Kanle-Modus prfen
		
		ldi		MDMX_Framecount_L, low(256)		; ja, Framecounter auf 256 setzen
		ldi		MDMX_Framecount_H, high(256)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_512:

		cpi		temp1, 0xA2						; mDMX-Header-Byte 512 Kanle empfangen...?
		brne	_MDMX_Header_End_Select			; nein, Fehler
		
		ldi		MDMX_Framecount_L, low(512)		; ja, Framecounter auf 512 setzen
		ldi		MDMX_Framecount_H, high(512)

		rjmp	_MDMX_Header_Finish

_MDMX_Header_End_Select:

		rjmp	_ISR_MDMX_Clear					; Kein gltiges Framesize-Byte -> Fehler

_MDMX_Header_Finish:

		ldi		MDMX_State, 102					; nchster Aufruf: Daten speichern

		ldi		ZH, high(MDMX_Buffer)			; Adresspointer auf Empfangspuffer setzen
		ldi		ZL, low(MDMX_Buffer)

		mov		MDMX_Buffer_L, ZL				; sichern
		mov		MDMX_Buffer_H, ZH

_dmx_send_wait_0:

		lds		temp0, UCSR1A					; warten auf Tx bereit
		sbrs	temp0, UDRE1
		rjmp	_dmx_send_wait_0

		clr		temp1							; DMX-Startbyte
		sts		UDR1, temp1						; verschicken

		rjmp	_ISR_MDMX_Ende					; und raus
	
; ------------------------------------------------------------------------------------------------
; 102 = Pixeldaten speichern
; ------------------------------------------------------------------------------------------------

_ISR_MDMX_Store_Byte:


		mov		ZL, MDMX_Buffer_L				; Pointer holen
		mov		ZH, MDMX_Buffer_H

		st		Z+, temp1						; empfangenes Byte speichern

_dmx_send_wait_1:

		lds		temp0, UCSR1A					; warten auf Tx bereit
		sbrs	temp0, UDRE1
		rjmp	_dmx_send_wait_1

		sts		UDR1, temp1						; empf. Byte per DMX weiterschicken

		mov		MDMX_Buffer_L, ZL				; Pointer wieder sichern
		mov		MDMX_Buffer_H, ZH

		subi	MDMX_Framecount_L, 1			; Framecounter runterzhlen
		sbci	MDMX_Framecount_H, 0			; in 16 Bit

		brne	_ISR_MDMX_Ende					; Frame noch nicht durch, Ende

		ldi		MDMX_State, 103					; Frame durch, warten auf Blockende

_dmx_send_wait_2:

		lds		temp0, UCSR1A					; warten auf letztes Byte durch
		sbrs	temp0, UDRE1
		rjmp	_dmx_send_wait_2

		clr		temp1					 		; Tx disable = Break
		sts		UCSR1B, temp1

		rjmp	_ISR_MDMX_Ende					; und raus

; ------------------------------------------------------------------------------------------------
; 103 = Warten auf Blockende
; ------------------------------------------------------------------------------------------------

_MDMX_Blockende:

		cpi		temp1, 0xA5						; Blockende-Byte empfangen...?
		brne	_ISR_MDMX_Clear					; nein, Fehler, keine Besttigung, Receiver Reset

		ldi		temp1, 1						; ja, Transmitter-Status: Ack Senden
		mov		tx_State, temp1
		ldi		temp1, 0x5A						; Blockstart-Byte Besttigung
		sts		UDR0, temp1						; Byte ausgeben (restliche Bytes in ISR)

		in		temp1,	S_LED_Port				; Status-LED toggeln
		ldi		temp0, (1<<S_LED)
		eor		temp1, temp0
		out		S_LED_Port, temp1

		sbr		Flags, (1<<FL_FRAME_REC)		; Flag fr Frame empfangen setzen

		ser		Timebase						; Timeout resetten

; ------------------------------------------------------------------------------------------------

_ISR_MDMX_Clear:

		clr		MDMX_State						; Receiver auf Anfang setzen

_ISR_MDMX_Ende:

		pop		ZH								; Register wiederherstellen
		pop		ZL
		pop		temp0
		pop		temp1
		out		SREG, temp1
		pop		temp1

		reti									; Ende Routine

; ================================================================================================
